@@ -1,10 +1,9 @@
 from __future__ import print_function
 
 import os
-import sys
 import json
 import ctypes
-from jsengine.util import to_bytes, json_encoder
+from jsengine.util import to_bytes
 
 try:
     from py_mini_racer import py_mini_racer
@@ -18,13 +17,8 @@ else:
 v8 = None
 
 injected_script = u'''\
-try {{
-    {code}
-    JSON.stringify([true, eval({expression})])
-}}
-catch (err) {{
-    JSON.stringify([false, err.toString()])
-}}
+{code}
+JSON.stringify([({expression})])
 '''
 
 try:
@@ -34,45 +28,36 @@ try:
 except ImportError:
     esprima = warned = False
 
-    def escheck(code):
-        return False
-
     def split_last_expr(code):
         global warned
         if not warned:
-            print('There is a scope issue with calss declarations in PyMiniRacer,'
+            import sys
+            print('There is a scope issue with class declarations in PyMiniRacer,'
                   ' measures to relieve it result in degrade performance!'
                   ' Please install esprima to resolve the issue.',
                   file=sys.stderr)
             warned = True
-        return u'', code
+        return u'', u'eval(%r)' % code
 else:
     if v8_available:
 
-        def escheck(code):
-            try:
-                Parser(code).parseScript()
-            except Exception:
-                return True # program errors
-            else:
-                return False
+        import re
+        _strip_ending = re.compile('[\s;]*$', re.UNICODE).sub
 
         def split_last_expr(code, options={'range': True}):
             try:
                 nodes = Parser(code, options=options).parseScript().body
             except Exception:
-                return u'', code  # program errors
+                return code, u''  # program errors
             nodes.reverse()
             start = len(code)
             for node in nodes:
-                if node.type == 'EmptyStatement':
-                    continue
-                if node.type != 'ExpressionStatement' or \
-                        node.expression.value == u'use strict':
+                if node.type == 'ExpressionStatement':
+                    start = node.range[0]
                     break
-                start = node.range[0]
-            return code[:start], code[start:]
-
+            code, expression = code[:start], code[start:]
+            expression = _strip_ending(u'', expression, 1)
+            return code, expression
 
 class MiniRacer(object):
 
@@ -83,17 +68,12 @@ class MiniRacer(object):
         self.ctx = v8.mr_init_context(b'--single-threaded')  # disable background
 
     def eval(self, code, raw=False):
-        jsonr = False
-        expression = u''
-        if not raw:
+        if raw:
+            expression = u''
+        else:
             code, expression = split_last_expr(code)
-        elif escheck(code):
-            # Get V8 errors instead of PyMiniRacer's Exception
-            code, expression = expression, code
-        if expression:
-            expression = json_encoder.encode(expression)
-            code = injected_script.format(code=code, expression=expression)
-            jsonr = True
+            if expression:
+                code = injected_script.format(code=code, expression=expression)
         code = to_bytes(code)
 
         res = v8.mr_eval_context(self.ctx, code, len(code),
@@ -109,10 +89,10 @@ class MiniRacer(object):
                 return True, None
             return False, e
         except (py_mini_racer.JSParseException, py_mini_racer.JSEvalException) as e:
-            return False, e
+            return False, e.args[0]
 
-        if jsonr:
-            return json.loads(res)
+        if expression:
+            return True, json.loads(res)[0]
 
         return True, None if raw else res
 
